@@ -35,6 +35,7 @@ Fields / columns binding
 
 """
 from csv2odoo import exceptions
+from csv2odoo.constants_and_vars import STATS
 
 
 class Field(object):
@@ -75,12 +76,12 @@ class Field(object):
     __slots__ = ['columns', 'default', 'name', 'relation', 'unique', 'callbacks',
                  'required', 'skip', 'ignore', 'replace', 'update', 'readonly',
                  'metadata', 'overwrite', 'concatenate', 'add', 'context',
-                 'init_state', 'string']
+                 'init_state', 'string', 'separator']
 
     def __init__(self, name, columns=None, default=None, callbacks=None,
                  relation=None, required=False, readonly=False, skip=False,
                  ignore=False, replace=False, update=False, unique=False,
-                 overwrite=True, concatenate=True, context=None):
+                 overwrite=True, separator=u',', concatenate=True, context=None):
 
         if context is None:
             context = {}
@@ -106,6 +107,7 @@ class Field(object):
             'unique': unique,
             'overwrite': overwrite,
             'concatenate': concatenate,
+            'separator': separator,
             'context': context,
         }
 
@@ -124,6 +126,7 @@ class Field(object):
         self.unique = unique
         self.overwrite = overwrite
         self.concatenate = concatenate
+        self.separator = separator
         self.context = context
 
         # Metadata
@@ -305,12 +308,19 @@ class Field(object):
         :returns: object
 
         """
+        global STATS
         def _apply_callbacks(value):
             # Apply callbacks successively
             if self.callbacks:
                 for callback in self.callbacks:
+                    session.log(
+                        u'debug', model, 
+                        u'"%s" before callback `%s`: %s' % (self.name, callback.__name__, value))
                     value = self._get_callback(
                         callback, session, model, value, line)
+                    session.log(
+                        u'debug', model, 
+                        u'"%s" after callback `%s`: %s' % (self.name, callback.__name__, value))
             return value
 
         def _get_columns_values():
@@ -322,7 +332,7 @@ class Field(object):
                         value = self.default
                     brut_values = [line[column] for column in self.columns]
                     if self.concatenate:
-                        brut_values = ','.join(brut_values)
+                        brut_values = self.separator.join(brut_values)
                     value = _apply_callbacks(brut_values)
                     if value and type(value) in (list, tuple):
                         value = u','.join(list(set(value)))
@@ -337,9 +347,12 @@ class Field(object):
                     session.log(
                         u'debug', model, u'Create relation `%s`...' % datatype)
                     for relation in self.relation:
-                        value = session.create(
-                            relation,
-                            relation.get_data(session, line))
+                        data = relation.get_data(session, line)
+                        if data:
+                            value = session.create(
+                                relation,
+                                data)
+                            STATS[u'objects'][relation.string]['ids'].append(value)
                     session.log(u'debug', model, u'Result: %s' % value)
 
                 elif datatype in ('one2many', 'many2many'):
@@ -349,11 +362,15 @@ class Field(object):
                     for m in self.relation:
                         # TODO: manage overwrite option
                         #if self.overwrite:
-                        id_ = session.create(m, m.get_data(session, line))
+                        data = m.get_data(session, line)
+                        id_ = 0
+                        if data:
+                            id_ = session.create(m, data)
                         if id_:
                             value.append(id_)
 
                         session.log(u'debug', model, u'Result: %s' % value)
+                        STATS[u'objects'][m.string]['ids'].extend(value)
 
                 else:  # datatype not in (u'one2one', u'one2many', u'many2one', u'many2many'):
                     raise exceptions.BadFieldType(model, self.name)
@@ -373,8 +390,6 @@ class Field(object):
 
         value = _get_columns_values()
         value = _get_relation_values(value)
-
-
         if datatype in ('many2one', 'one2one'):
 
             session.log(u'debug', model, u'Relation `%s`...' % datatype)
@@ -400,10 +415,11 @@ class Field(object):
 
         # If skippable
         if self.skip and not value:
-            raise exceptions.SkipLineException(u"Line skipped")
+            raise exceptions.SkipLineException(u"field '%s' has value '%s'" % (self.name, value))
         # If ignorable
         if self.ignore and not value:
-            raise exceptions.SkipObjectException(u"Model skipped")
+            session.log(u'info', model, u'%s: %s' % (self.name, value))
+            raise exceptions.SkipObjectException(u"field '%s' has value '%s'" % (self.name, value))
 
         # Manage null values
         #===================================================================
@@ -428,162 +444,16 @@ class Field(object):
         return value
 
     def __str__(self):
-        return u"<Field %s from columns %s>" % (self.field, self.columns)
+        from_ = self.columns or (self.relation and [r.name for r in self.relation])
+        return u"<Field `%s` => %s>" % (self.name, from_)
+
+    def __repr__(self):
+        return str(self)
 
     def __getitem__(self, item):
         return getattr(self, item)
 
 
-class Suite(Field):
-    """ Allow to treat each separated values of the field as an unique object.
-
-    :param separator: The separator used to isolate each values
-    :type separator: char
-
-    """
-
-    __slots__ = ['column', 'default', 'field', 'unique', 'callback', 'search',
-                 'required', 'skip', 'ignore', 'replace', 'update', 'separator']
-
-    def __init__(self, column, separator, callback):
-        self.column = column
-        self.default = None
-        self.field = None
-        self.callback = callback
-        self.search = False
-        self.required = False
-        self.separator = u',' 
-        self.skip = False
-        self.ignore = False
-        self.replace = False
-        self.update = False
-        self.unique = False
-
-    def get_data(self, session, model, line):
-        """Return the value relative to the line in argument.
-
-        :param session: An instance of :class:`csv2odoo.Session`
-        :param model: An instance of :class:`csv2odoo.Model`
-        :param line: The line to search for
-        :type line: list
-        :raises: SkipLineException, SkipObjectException, RequiredFieldError
-        :returns: object
-
-        """
-        value = super(Suite, self).get_data(session, model, line)
-        print self.name, value
-        # Retrieving field metadatas
-        self.get_metadata(session, model)
-        try:
-            datatype = self.metadata['type']
-        except:
-            datatype = None
-
-        if self.columns:
-            # TODO: gérer chaque valeur de chaque colonne indépendamment
-            try:
-                value = [line[c] for c in self.columns]
-            except IndexError:
-                session.log(u'error', model, u'Columns `%s` mismatch, out of range.' % (self.columns))
-
-        elif self.relation:
-
-            if datatype in ('many2one', 'one2one'):
-                session.log(
-                    u'debug', model, u'Create relation `%s`...' % datatype)
-                for relation in self.relation:
-                    value = session.create(
-                        relation,
-                        relation.get_data(session, line))
-                session.log(u'debug', model, u'Result: %s' % value)
-
-            elif datatype in ('one2many', 'many2many'):
-                if self.overwrite:
-                    session.log(u'debug', model,
-                                u'Create relation `%s`...' % datatype)
-                    ids = []
-                    for m in self.relation:
-                        id_ = session.create(m, m.get_data(session, line))
-                        if id_:
-                            ids.append(id_)
-                    if ids:
-                        value = [(6, 0, ids)]
-                    session.log(u'debug', model, u'Result: %s' % value)
-
-                else:
-                    value = []
-                    session.log(u'debug', model,
-                                u'Create relation `%s`...' % datatype)
-                    for m in self.relation:
-                        id_ = session.create(m, m.get_data(session, line))
-                        if id_:
-                            value.append(id_)
-                    session.log(u'debug', model, u'Result: %s' % value)
-
-            else:  # datatype not in (u'one2one', u'one2many', u'many2one', u'many2many'):
-                raise exceptions.BadFieldType(model, self.name)
-
-        # Apply callbacks successively
-        if self.callbacks:
-            for callback in self.callbacks:
-                value = self._get_callback(
-                    callback, session, model, value, line)
-
-        if datatype in ('many2one', 'one2one'):
-
-            session.log(u'debug', model, u'Relation `%s`...' % datatype)
-            session.log(u'debug', model, u'Result: %s' % value)
-
-        elif datatype in ('one2many', 'many2many'):
-
-            if self.overwrite:
-                session.log(u'debug', model, u'Relation `%s`...' % datatype)
-                ids = []
-                value = [(6, 0, value)]
-                session.log(u'debug', model, u'Result: %s' % value)
-
-            else:
-                values = []
-                session.log(u'debug', model, u'Relation `%s`...' % datatype)
-                for m in value:
-                    values.append(
-                        #(4, session.create(m, m.get_data(session, line))),)
-                        (4, m),)
-                value = values
-                session.log(u'debug', model, u'Result: %s' % value)
-
-        # If skippable
-        if self.skip and not value:
-            raise exceptions.SkipLineException(u"Line skipped")
-        # If ignorable
-        if self.ignore and not value:
-            raise exceptions.SkipObjectException(u"Model skipped")
-        # If default
-        if self.default and not value:
-            value = self.default
-
-        # Manage null values
-        #===================================================================
-        try:
-            value.strip(u' \t\n\r\r\n')
-        except:
-            pass
-
-        #Cleaning dict items with null value and ignored field
-        #if data not in (0, '0', False, True) and not data[field]:
-        #    if self.required:
-        #        raise exceptions.RequiredFieldError(
-        #            u'%s %s field is required. Skipped.' % (model, field))
-        #    ToRemove_fields.append(self.name)
-        #if self.router:
-        #    self.router(value)
-        #    del data
-        return value
-
-    def __str__(self):
-        return u"<Suite %s from columns %s>" % (self.field, self.columns)
-
-
 class Router(Field):
     """ Specify the column number and special treatments from which the current
     model's field will be allocated to the object's creation.
@@ -615,4 +485,4 @@ class Router(Field):
         pass
 
     def __str__(self):
-        return u"<Router %s from columns %s>" % (self.field, self.columns)
+        return u"<Router %s from columns %s>" % (self.name, self.columns)
