@@ -56,6 +56,7 @@ try:
 except ImportError:
     progressbar = None
 
+from csv2odoo.parser import OPTS, ARGS
 from csv2odoo import exceptions
 from csv2odoo import tools
 from csv2odoo.constants_and_vars import STATS, FORMAT
@@ -70,7 +71,7 @@ QUOTING_MAPPING = {
 }
 
 
-class Session(object):
+class Csv(object):
     """Main class which provides the functionnal part of the importation process.
 
     .. note:: `sys.argv` integrated provides a command line parser.
@@ -103,12 +104,12 @@ class Session(object):
         """
         return deepcopy(list(self._lines()))
 
-    def __init__(self, name, filename, delimiter=u';', quotechar=u'"',
+    def __init__(self, filename, name=None, delimiter=u';', quotechar=u'"',
                  encoding=u'utf-8', offset=0, limit=None, quiet=False, debug=False,
                  quoting="all", **kwargs):
 
         # Unique id of this import
-        self.name = unicode(name)
+        self.name = unicode(name or filename)
         self.id = unicode(time.time())
 
         # Pass args to sys.argv
@@ -121,93 +122,13 @@ class Session(object):
         self._encoding = u'utf-8'
         self._logger = None
 
-        parser = OptionParser()
-        parser.add_option(u"-f", u"--filename",
-                          action=u"store",
-                          dest=u"filename",
-                          default=filename,
-                          help=u"The CSV file to import.")
-        parser.add_option(u"-s", u"--separator",
-                          action=u"store",
-                          dest=u"delimiter",
-                          default=delimiter,
-                          help=u"The delimiter of the CSV file.")
-        parser.add_option(u"-g", u"--quotechar",
-                          action=u"store",
-                          dest=u"quotechar",
-                          default=quotechar,
-                          help=u"The quotechar of the CSV file.")
-        parser.add_option(u"-n", u"--quoting",
-                          action=u"store",
-                          dest=u"quoting",
-                          default=quoting,
-                          help=u"The quoting of the CSV file.")
-        parser.add_option(u"-e", u"--encoding",
-                          action=u"store",
-                          dest=u"encoding",
-                          default=encoding,
-                          help=u"The encoding of the CSV file.")
-        parser.add_option(u"-o", u"--offset",
-                          action=u"store",
-                          type=u"int",
-                          dest=u"offset",
-                          default=offset,
-                          help=u"Offset (Usually used for header omission, default=1)")
-        parser.add_option(u"-d", u"--debug",
-                          action=u"store_true",
-                          dest=u"debug",
-                          default=False,
-                          help=u"debug mode")
-        parser.add_option(u"-v", u"--verbose",
-                          action=u"store_true",
-                          dest=u"verbose",
-                          default=debug,
-                          help=u"verbose mode")
-        parser.add_option(u"-q", u"--quiet",
-                          action=u"store_true",
-                          dest=u"quiet",
-                          default=quiet,
-                          help=u"don't print anything to stdout")
-        parser.add_option(u"-l", u"--limit",
-                          action=u"store",
-                          dest=u"limit",
-                          type=u"int",
-                          default=limit,
-                          help=u"Limit")
-        # Connection options
-        parser.add_option(u"-u", u"--username",
-                          action=u"store",
-                          dest=u"username",
-                          default=u'admin',
-                          help=u"Username")
-        parser.add_option(u"-p", u"--password",
-                          action=u"store",
-                          dest=u"password",
-                          default=u'admin',
-                          help=u"Password")
-        parser.add_option(u"-r", u"--host",
-                          action=u"store",
-                          dest=u"host",
-                          default=u'localhost',
-                          help=u"Host to contact")
-        parser.add_option(u"-t", u"--port",
-                          action=u"store",
-                          dest=u"port",
-                          default=8069,
-                          help=u"Port used")
-        parser.add_option(u"-b", u"--database",
-                          action=u"store",
-                          dest=u"dbname",
-                          default=u'test',
-                          help=u"Database name")
-        (self._opts, args) = parser.parse_args()
-        self.filename = self._opts.filename
-        self.delimiter = self._opts.delimiter
-        self.quotechar = self._opts.quotechar
-        self.quoting = self._opts.quoting
-        self.encoding = self._opts.encoding
-        self.offset = self._opts.offset
-        self.limit = self._opts.limit
+        self.filename = filename or OPTS.filename
+        self.delimiter = delimiter or OPTS.delimiter
+        self.quotechar = quotechar or OPTS.quotechar
+        self.quoting = quoting or OPTS.quoting
+        self.encoding = encoding or OPTS.encoding
+        self.offset = offset or OPTS.offset
+        self.limit = limit or OPTS.limit
         self.models = []
         self.server = None
 
@@ -340,15 +261,15 @@ class Session(object):
         self._hdlr.setFormatter(formatter)
         self._logger.addHandler(self._hdlr)
 
-        if not self._opts.quiet or quiet:
+        if not quiet or not OPTS.quiet:
             # In all times, show WARNING and ERROR messages
             logging.basicConfig(format=FORMAT)
             self._logger.setLevel(logging.WARNING)
 
-            if self._opts.verbose or verbose:
+            if verbose or OPTS.verbose:
                 self._logger.setLevel(logging.INFO)
 
-            if self._opts.debug or debug:
+            if debug or OPTS.debug:
                 self._logger.setLevel(logging.DEBUG)
 
         return True
@@ -463,7 +384,7 @@ class Session(object):
 
         res = []
         list_search = []
-        self._logger.debug(u'<%-25s> search:%s' % (model.name, model.search),
+        self._logger.debug(u'<%-25s> search:%s' % (model.string, model.search),
                            extra=STATS[self.id])
 
         for item in model.search:
@@ -471,7 +392,7 @@ class Session(object):
                 list_search.append((item, u'=', data[item]))
 
         self._logger.debug(
-            u'<%-25s> list_search:%s' % (model.name, list_search),
+            u'<%-25s> list_search:%s' % (model.string, list_search),
             extra=STATS[self.id])
 
         for (item, sign, val) in copy(list_search):
@@ -482,7 +403,7 @@ class Session(object):
         if list_search:
             res = self.server.search(model.name, list_search, 0, False,
                                      False, {u'lang': self._lang})
-            self._logger.debug(u'<%-25s> search result %s' % (model.name, res),
+            self._logger.debug(u'<%-25s> search result %s' % (model.string, res),
                                extra=STATS[self.id])
 
         return res
@@ -501,6 +422,7 @@ class Session(object):
         :param search: List of field used to search for an existing object
         :type search: list
         :returns: int
+        :raises: SkipObjectException
 
         """
         global STATS
@@ -518,14 +440,12 @@ class Session(object):
             if data[attr] in ('', None):
                 del data[attr]
 
-        self._logger.debug(u'<%-25s> data:%s' % (model.name, data),
+        self._logger.debug(u'<%-25s> data:%s' % (model.string, data),
                            extra=STATS[self.id])
 
         # If data is empty SKIPPED
         if not data:
-            self._logger.warning(u"<%-25s> Null values" % model.name,
-                                 extra=STATS[self.id])
-            STATS[u'objects'][model.name][u'skipped'] += 1
+            raise exceptions.SkipObjectException("Null values")
 
         else:
             try:
@@ -545,14 +465,14 @@ class Session(object):
                         _id = self.server.create(
                             model.name, data, context={u'lang': self._lang})
                         self._logger.info(
-                            u"<%-25s> CREATE [%s]" % (model.name, _id),
+                            u"<%-25s> CREATE [%s]" % (model.string, _id),
                             extra=STATS[self.id])
                         STATS[self.id][u'object_created'] += 1
-                        STATS[u'objects'][model.name][u'created'] += 1
+                        STATS[u'objects'][model.string][u'created'] += 1
                     else:
                         self._logger.info(
                             u"<%-25s> NO CREATE [%s]" % (
-                                model.name, _id or name),
+                                model.string, _id or name),
                             extra=STATS[self.id])
 
                 else:
@@ -560,13 +480,13 @@ class Session(object):
                     if model.update:
                         self.server.write(model.name, [_id], data)
                         self._logger.info(
-                            u"<%-25s> UPDATE [%s]" % (model.name, _id),
+                            u"<%-25s> UPDATE [%s]" % (model.string, _id),
                             extra=STATS[self.id])
                         STATS[self.id][u'object_written'] += 1
-                        STATS[u'objects'][model.name][u'written'] += 1
+                        STATS[u'objects'][model.string][u'written'] += 1
                     else:
                         self._logger.info(
-                            u"<%-25s> NO UPDATE [%s]" % (model.name, _id),
+                            u"<%-25s> NO UPDATE [%s]" % (model.string, _id),
                             extra=STATS[self.id])
 
             except xmlrpclib.Fault as err:
@@ -584,14 +504,14 @@ class Session(object):
             error = error.replace(u'\\n', u'\n').replace(u'\\', u'')
             try:
                 self._logger.error(u"<%-25s> id:%s %s" %
-                                   (model.name, _id, error), extra=STATS[self.id])
-                self._logger.debug(u"<%-25s> id:%s|state:%s|%s" %
-                                   (model.name, _id, state, pformat(data)), extra=STATS[self.id])
+                                   (model.string, _id, error), extra=STATS[self.id])
+                self._logger.error(u"<%-25s> id:%s|state:%s|%s" %
+                                   (model.string, _id, state, pformat(data)), extra=STATS[self.id])
             except:
                 print u'Erreur OPENERP :: %s' % error
             STATS[self.id][u'warnings'] += 1
-            STATS[u'objects'][model.name][u'skipped'] += 1
-            STATS[u'objects'][model.name][u'error'] += 1
+            STATS[u'objects'][model.string][u'skipped'] += 1
+            STATS[u'objects'][model.string][u'error'] += 1
 
         return _id
 
@@ -604,14 +524,19 @@ class Session(object):
         .. versionadded: 0.6
 
         """
-        if model.name not in STATS[u'objects']:
-            STATS[u'objects'][model.name] = {
-                u'name': model.name,
+        if model.string not in STATS[u'objects']:
+            STATS[u'objects'][model.string] = {
+                u'name': model.string,
+                u'model': model.name,
+                u'ids': [],
+                u'methods': model.methods or [],
                 u'written': 0,
                 u'created': 0,
                 u'skipped': 0,
                 u'error': 0,
             }
+        else:
+            STATS[u'objects'][model.string]['methods'].extend(model.methods or [])
 
     def _parse_models(self, line):
         """Start model importation.
@@ -622,13 +547,14 @@ class Session(object):
 
         """
         for model in self.models:
+            data = {}
 
             state = u'Unknown'
             search = u'Unknown'
             try:
-
                 data = model.get_data(self, line)
-                self.create(model, data)
+                id = self.create(model, data)
+                STATS[u'objects'][model.string]['ids'].append(id)
                 model.reinit()
 
             except exceptions.RequiredFieldError as error:
@@ -636,27 +562,27 @@ class Session(object):
                 continue
 
             except exceptions.SkipObjectException as error:
-                self._logger.info(u"<%-20s> Object skipped" %
-                                  model.name, extra=STATS[self.id])
-                STATS[u'objects'][model.name][u'skipped'] += 1
+                self._logger.info(u"<%-20s> Object skipped(%s)" %
+                                  (model.string, error), extra=STATS[self.id])
+                STATS[u'objects'][model.string][u'skipped'] += 1
                 STATS[self.id][u'object_skipped'] += 1
                 # Next object
                 continue
 
             except exceptions.SkipLineException as error:
-                self._logger.info(u"<%-20s> Line skipped(%s)" %
-                                  (model.name, error), extra=STATS[self.id])
+                self._logger.info(u"<%-20s> Line skipped (%s)" %
+                                  (model.string, error), extra=STATS[self.id])
                 STATS[self.id][u'line_skipped'] += 1
                 # Next line
                 break
 
             except exceptions.ColumnIndexError as error:
                 STATS[self.id][u'errors'] += 1
-                raise exceptions.FatalException(error, line, model.name, state, search, data, self.lines[line][1:])
+                raise exceptions.FatalException(error, line, model.string, state, search, data, self.lines[line][1:])
 
             #except Exception as error:
             #    STATS[self.id][u'errors'] += 1
-            #    raise exceptions.FatalException(error, line, model.name, state, search, data, self.lines[line][1:])
+            #    raise exceptions.FatalException(error, line, model.string, state, search, data, self.lines[line][1:])
 
             # Keep line incrementation
             STATS[self.id][u'line_done'] = STATS[self.id][u'line_num']
@@ -664,6 +590,24 @@ class Session(object):
             #===========================================================
             #self._logger.info(u"next line", extra=STATS[self.id])
 
+    def _call_methods(self):
+        """ Call method defined in the model definition
+
+        """
+        for string, values in STATS[u'objects'].iteritems():
+            if not values['ids']:
+                continue
+            ids = list(set(values['ids']))
+            for method in values['methods']:
+                self._logger.info(
+                    u"Calling methods '%s.%s(%s)'..." %
+                    (values["model"], method, ids),
+                    extra=STATS[self.id])
+                for id in ids:
+                    self.server.execute(values["model"], method, [id])
+
+            values['ids'] = []
+
     def _launch_import(self, *args, **kwargs):
         """Main function that launch the importation process
 
@@ -671,7 +615,7 @@ class Session(object):
         global STATS
         exit = False
         STATS[self.id][u'importfile'] = self.filename
-        if not self._opts.quiet:
+        if not OPTS.quiet:
             sys.stdout.write(u'\n')
             sys.stdout.write(u'*' * 79 + u'\n')
             sys.stdout.write(u'* %s\n' % self.name)
@@ -686,13 +630,14 @@ class Session(object):
         # Open file, call preconfigure if given in arg
         #=======================================================================
         try:
-            if progressbar and not self._opts.quiet and not self._opts.verbose:
+            total_lines = self.lines[self.offset:self.limit]
+            if progressbar and not OPTS.quiet and not OPTS.verbose:
                 pbar = ProgressBar(widgets=[
                     u'[', self.filename, u']',
                     u'Processing line ', SimpleProgress(),
                     u' (', Percentage(), u' | ', ETA(), u' | ', FileTransferSpeed(), u')'
-                ], maxval=len(self.lines) + 1).start()
-            for line in self.lines[self.offset:self.limit]:
+                ], maxval=len(total_lines) + 1).start()
+            for line in total_lines:
                 # Keeping stats
                 STATS[self.id][u'line_num'] += 1
                 STATS[self.id][u'actual_line'] = line
@@ -701,14 +646,16 @@ class Session(object):
                 # from CSV line according to columns mapping
                 self._parse_models(line)
 
-                if progressbar and not self._opts.quiet and not self._opts.verbose:
+                if progressbar and not OPTS.quiet and not OPTS.verbose:
                     pbar.update(int(STATS[self.id][u'line_num']))
 
                 # Flush models
                 for model in self.models:
                     model.reinit()
 
-            if progressbar and not self._opts.quiet and not self._opts.verbose:
+                self._call_methods()
+
+            if progressbar and not OPTS.quiet and not OPTS.verbose:
                 pbar.finish()
 
             self._logger.info(u"END import", extra=STATS[self.id])
@@ -722,7 +669,7 @@ class Session(object):
             exit = True
 
         finally:
-            if not self._opts.quiet:
+            if not OPTS.quiet:
                 sys.stdout.write(u'*' * 79 + u'\n')
                 sys.stdout.write(u'Object Skipped %(object_skipped)11s' %
                                  STATS[self.id])
@@ -780,7 +727,7 @@ class Session(object):
         if not self._logger:
             self.set_logger()
         getattr(self._logger, level)(u"%s%s" % (model and u'<%-25s> ' %
-                                                model.name or u'', msg), extra=STATS[self.id])
+                                                model.string or u'', msg), extra=STATS[self.id])
         STATS[self.id][u'line_num'] = old_line
 
     @staticmethod
@@ -803,9 +750,9 @@ class Session(object):
         """
         global STATS
         if server is None:
-            server = Odoo(self._opts.host, self._opts.port,
-                             self._opts.username, self._opts.password,
-                             self._opts.dbname)
+            server = Odoo(OPTS.host, OPTS.port,
+                             OPTS.username, OPTS.password,
+                             OPTS.dbname)
         self.server = server
         self._lang = server.lang
         self._check_server(server)
